from abc import ABC, abstractmethod
from os import PathLike
from pathlib import Path
from typing import TYPE_CHECKING, Any

from ..caster import cast

if TYPE_CHECKING:
    from diot import Diot


class Loader(ABC):

    CASTERS = None

    @abstractmethod
    def loading(self, conf: Any, ignore_nonexist: bool) -> "Diot":
        """Load the configuration from the path or configurations"""

    def _exists(self, conf: PathLike, ignore_exist: bool) -> bool:
        """Check if the configuration file exists"""
        path = Path(conf)
        if not ignore_exist and not path.exists():
            raise FileNotFoundError(f"{conf} does not exist")
        return path.exists()

    def load(self, conf: Any, ignore_nonexist: bool = False) -> "Diot":
        """Load the configuration from the path or configurations and cast
        values

        Args:
            conf: The configuration file to load

        Returns:
            The Diot object
        """
        loaded = self.loading(conf, ignore_nonexist)
        if self.__class__.CASTERS:
            return cast(loaded, self.__class__.CASTERS)
        return loaded

    load_with_profiles = load



from typing import Any
from simpleconf.utils import require_package
from diot import Diot

from . import Loader

yaml = require_package("yaml")


class YamlLoader(Loader):
    """Yaml file loader"""

    def loading(self, conf: Any, ignore_nonexist: bool) -> Diot:
        """Load the configuration from a yaml file"""
        if not self._exists(conf, ignore_nonexist):
            return Diot()

        with open(conf) as f:
            return Diot(yaml.load(f, Loader=yaml.FullLoader))



from contextlib import contextmanager
from typing import Any, List

from diot import Diot

from .utils import config_to_ext, get_loader, POOL_KEY, META_KEY


class Config:
    """The configuration class"""

    @staticmethod
    def load(*configs, ignore_nonexist: bool = False) -> Diot:
        """Load the configuration from the files, or other configurations

        Args:
            *configs: The configuration files or other configurations to load
                Latter ones will override the former ones for items with the
                same keys recursively.
            ignore_nonexist: Whether to ignore non-existent files
                Otherwise, will raise errors

        Returns:
            A Diot object with the loaded configurations
        """
        out = Diot()
        for conf in configs:
            ext = config_to_ext(conf)
            loader = get_loader(ext)
            loaded = loader.load(conf, ignore_nonexist)
            out.update(loaded)

        return out

    @staticmethod
    def load_one(
        config, loader: str = None, ignore_nonexist: bool = False
    ) -> Diot:
        """Load the configuration from the file

        Args:
            config: The configuration file to load
            loader: The loader to use
            ignore_nonexist: Whether to ignore non-existent files
                Otherwise, will raise errors

        Returns:
            A Diot object with the loaded configuration
        """
        if loader is None:
            ext = config_to_ext(config)
            loader = get_loader(ext)
        else:
            loader = get_loader(loader)

        return loader.load(config, ignore_nonexist)


class ProfileConfig:
    """The configuration class with profile support"""

    @staticmethod
    def load(*configs: Any, ignore_nonexist: bool = False) -> Diot:
        """Load the configuration from the files, or other configurations

        Args:
            *configs: The configuration files or other configurations to load
                Latter ones will override the former ones for items with the
                same profile and keys recursively.
            ignore_nonexist: Whether to ignore non-existent files
                Otherwise, will raise errors
        """
        out = Diot({POOL_KEY: Diot()})
        pool = out[POOL_KEY]
        out[META_KEY] = {
            "current_profile": None,
            "base_profile": None,
        }
        for conf in configs:
            ext = config_to_ext(conf)
            loader = get_loader(ext)
            loaded = loader.load_with_profiles(conf, ignore_nonexist)
            for profile, value in loaded.items():
                profile = profile.lower()
                pool.setdefault(profile, Diot())
                pool[profile].update(value)

        ProfileConfig.use_profile(out, "default")
        return out

    @staticmethod
    def load_one(
        conf: Any, loader: str = None, ignore_nonexist: bool = False
    ) -> Diot:
        """Load the configuration from the file

        Args:
            conf: The configuration file to load
            loader: The loader to use. Will detect from conf by default
            ignore_nonexist: Whether to ignore non-existent files
                Otherwise, will raise errors

        Returns:
            A Diot object with the loaded configuration
        """

        out = Diot({POOL_KEY: Diot()})
        pool = out[POOL_KEY]
        out[META_KEY] = {
            "current_profile": None,
            "base_profile": None,
        }

        if loader is None:
            ext = config_to_ext(conf)
            loader = get_loader(ext)
        else:
            loader = get_loader(loader)

        loaded = loader.load_with_profiles(conf, ignore_nonexist)
        for profile, value in loaded.items():
            profile = profile.lower()
            pool.setdefault(profile, Diot())
            pool[profile].update(value)

        ProfileConfig.use_profile(out, "default")
        return out

    @staticmethod
    def use_profile(
        conf: Diot,
        profile: str,
        base: str = "default",
        copy: bool = False,
    ) -> Diot:
        """Switch the configuration to the given profile, based on the
        default profile.

        Args:
            conf: The configuration object by the `load` function
            profile: The profile to use
            default: The default profile

        Returns:
            The configuration object with the switched profile if copy is True
            Otherwise None (updated in-place)
        """
        pool = conf[POOL_KEY]
        if copy:
            out = Diot({POOL_KEY: pool, META_KEY: conf[META_KEY].copy()})
            if base is not None:
                out.update(pool[base])
            out[META_KEY]["current_profile"] = profile
            out[META_KEY]["base_profile"] = base
            out.update(pool[profile])
            return out

        # copy = False
        for key in list(conf):
            if key in (POOL_KEY, META_KEY):
                continue
            del conf[key]

        if base is not None:
            conf.update(pool[base])
        conf.update(pool[profile])
        conf[META_KEY]["current_profile"] = profile
        conf[META_KEY]["base_profile"] = base

    @staticmethod
    def current_profile(conf: Diot) -> str:
        """Get the current profile"""
        return conf[META_KEY]["current_profile"]

    @staticmethod
    def base_profile(conf: Diot) -> str:
        """Get the base profile"""
        return conf[META_KEY]["base_profile"]

    @staticmethod
    def detach(conf: Diot) -> Diot:
        """Detach the configurations of current profile from the
        configuration object.
        Profile information will be removed.

        Args:
            conf: The configuration object by the `load` function

        Returns:
            The configurations with the current profile
        """
        out = Diot()
        for key in conf:
            if key in (POOL_KEY, META_KEY):
                continue
            out[key] = conf[key]
        return out

    @staticmethod
    def pool(conf: Diot) -> Diot:
        """Get the pool"""
        return conf[POOL_KEY]

    @staticmethod
    def profiles(conf: Diot) -> List:
        """Get the profiles in the configuration

        Args:
            conf: The configuration object by the `load` function

        Returns:
            The list of profiles
        """
        return list(conf[POOL_KEY])

    @staticmethod
    def has_profile(conf: Diot, profile: str) -> bool:
        """Check if the configuration has the given profile

        Args:
            conf: The configuration object by the `load` function
            profile: The profile to check

        Returns:
            Whether the configuration has the given profile
        """
        return profile in conf[POOL_KEY]

    @staticmethod
    @contextmanager
    def with_profile(conf: Diot, profile: str, base: str = "default") -> Diot:
        """A context manager to use the given profile

        Args:
            conf: The configuration object by the `load` function
            profile: The profile to use
            base: The base profile

        Yields:
            The configuration object with the switched profile
        """
        prev_profile = ProfileConfig.current_profile(conf)
        prev_base = ProfileConfig.base_profile(conf)
        ProfileConfig.use_profile(conf, profile, base)
        yield conf
        ProfileConfig.use_profile(conf, prev_profile, prev_base)



from os import environ
from typing import Any
import warnings

from diot import Diot

from . import Loader
from ..caster import (
    cast,
    int_caster,
    float_caster,
    bool_caster,
    none_caster,
    python_caster,
    py_caster,
    json_caster,
    toml_caster,
)


class OsenvLoader(Loader):
    """Environment variable loader"""

    CASTERS = [
        int_caster,
        float_caster,
        bool_caster,
        none_caster,
        python_caster,
        py_caster,
        json_caster,
        toml_caster,
    ]

    def loading(self, conf: Any, ignore_nonexist: bool = False) -> Diot:
        """Load the configuration from environment variables"""
        prefix = f"{conf[:-6]}_" if len(conf) > 6 else ""
        len_prefix = len(prefix)
        out = Diot()
        for k, v in environ.items():
            if k.startswith(prefix):
                out[k[len_prefix :]] = v
        return out

    def load_with_profiles(
        self,
        conf: Any,
        ignore_nonexist: bool = False,
    ) -> Diot:
        prefix = f"{conf[:-6]}_" if len(conf) > 6 else ""
        len_prefix = len(prefix)
        out = Diot()
        for k, v in environ.items():
            if not k.startswith(prefix):
                continue
            key = k[len_prefix:]
            if "_" not in key:
                warnings.warn(f"{conf}: No profile name found in key: {k}")
                continue
            profile, key = key.split("_", 1)
            profile = profile.lower()
            out.setdefault(profile, Diot())[key] = v

        return cast(out, self.__class__.CASTERS)



from typing import Any
from diot import Diot

from . import Loader


class DictLoader(Loader):
    """Dict loader"""

    def loading(self, conf: Any, ignore_nonexist: bool) -> Diot:
        """Load the configuration from a dict"""
        return Diot(conf)



import warnings
from pathlib import Path
from typing import Any
from simpleconf.utils import require_package
from diot import Diot

from ..caster import (
    cast,
    int_caster,
    float_caster,
    bool_caster,
    none_caster,
    python_caster,
    py_caster,
    json_caster,
    toml_caster,
)
from . import Loader

dotenv = require_package("dotenv")


class EnvLoader(Loader):
    """Env file loader"""

    CASTERS = [
        int_caster,
        float_caster,
        bool_caster,
        none_caster,
        python_caster,
        py_caster,
        json_caster,
        toml_caster,
    ]

    def loading(self, conf: Any, ignore_nonexist: bool = False) -> Diot:
        """Load the configuration from a .env file"""
        if not self._exists(conf, ignore_nonexist):
            return Diot()
        return Diot(dotenv.main.DotEnv(conf).dict())

    def load_with_profiles(
        self,
        conf: Any,
        ignore_nonexist: bool = False,
    ) -> Diot:
        """Load and cast the configuration from a .env file with profiles"""
        envs = self.loading(conf, ignore_nonexist)
        out = Diot()
        for k, v in envs.items():
            if "_" not in k:
                warnings.warn(
                    f"{Path(conf).name}: No profile name found in key: {k}"
                )
                continue
            profile, key = k.split("_", 1)
            profile = profile.lower()
            out.setdefault(profile, Diot())[key] = v

        return cast(out, self.__class__.CASTERS)



from typing import Any
from simpleconf.utils import require_package
from diot import Diot

from ..caster import (
    none_caster,
    null_caster,
)
from . import Loader

rtoml = require_package("rtoml")


class TomlLoader(Loader):
    """Toml file loader"""

    CASTERS = [
        none_caster,
        null_caster,
    ]

    def loading(self, conf: Any, ignore_nonexist: bool) -> Diot:
        """Load the configuration from a toml file"""
        if not self._exists(conf, ignore_nonexist):
            return Diot()

        with open(conf) as f:
            return Diot(rtoml.load(f))




from pathlib import Path
from importlib import import_module
from types import ModuleType
from typing import TYPE_CHECKING, Any

from .exceptions import FormatNotSupported

if TYPE_CHECKING:
    from .loaders import Loader

POOL_KEY = "_SIMPLECONF_POOL"
META_KEY = "_SIMPLECONF_META"


def config_to_ext(conf: Any) -> str:
    """Find the extension(flag) of the configuration"""
    if isinstance(conf, dict):
        return "dict"

    conf = Path(conf)
    out = conf.suffix.lstrip(".").lower()
    if not out and conf.name.lower().endswith("rc"):
        out = "rc"

    if out in ("ini", "rc", "cfg", "conf", "config"):
        return "ini"

    if out == "yml":
        return "yaml"

    return out


def get_loader(ext: str) -> "Loader":
    """Get the loader for the extension"""
    if ext == "dict":
        from .loaders.dict import DictLoader
        return DictLoader()
    if ext == "env":
        from .loaders.env import EnvLoader
        return EnvLoader()
    if ext == "ini":
        from .loaders.ini import IniLoader
        return IniLoader()
    if ext == "json":
        from .loaders.json import JsonLoader
        return JsonLoader()
    if ext == "osenv":
        from .loaders.osenv import OsenvLoader
        return OsenvLoader()
    if ext == "toml":
        from .loaders.toml import TomlLoader
        return TomlLoader()
    if ext == "yaml":
        from .loaders.yaml import YamlLoader
        return YamlLoader()

    raise FormatNotSupported(f"{ext} is not supported.")


def require_package(package: str) -> "ModuleType":
    """Require the package and return the module"""
    try:
        return import_module(package)
    except ModuleNotFoundError:
        raise ImportError(f"{package} is not installed.")



import json
from typing import Any

from diot import Diot

from . import Loader


class JsonLoader(Loader):
    """Json file loader"""

    def loading(self, conf: Any, ignore_nonexist: bool) -> Diot:
        """Load the configuration from a json file"""
        if not self._exists(conf, ignore_nonexist):
            return Diot()
        with open(conf) as f:
            return Diot(json.load(f))



import json
from typing import TYPE_CHECKING, Any, Callable, Sequence
from ast import literal_eval

from simpleconf.utils import require_package

if TYPE_CHECKING:
    from diot import Diot


def type_caster(prefix: str, cast_fun: Callable) -> Any:
    """Base type caster"""

    def _caster(value: str, fail_raises: bool = True) -> Any:
        """Cast value to base type"""
        if not value.startswith(prefix):
            if fail_raises:
                raise ValueError(f"Expect `@int:` prefix, got `{value}`")
            return value
        to_be_casted = value[len(prefix) :]
        try:
            return cast_fun(to_be_casted)
        except Exception:
            if fail_raises:
                raise
            return value

    return _caster


def _cast_none(value: str) -> None:
    """Cast None"""
    if value == "":
        return None

    raise ValueError(f"Expect `@none`, got `{value}`")


def _cast_bool(value: str) -> bool:
    """Cast bool"""
    if value.lower() == "true":
        return True
    if value.lower() == "false":
        return False
    raise ValueError(f"Expect `@bool:true` or `@bool:false`, got `{value}`")


def _cast_toml(value: str) -> Any:
    """Cast toml string"""
    rtoml = require_package("rtoml")
    return rtoml.loads(value)


int_caster = type_caster("@int:", lambda x: int(float(x)))
float_caster = type_caster("@float:", float)
bool_caster = type_caster("@bool:", _cast_bool)
none_caster = type_caster("@none", _cast_none)
null_caster = type_caster("null", _cast_none)
python_caster = type_caster("@python:", literal_eval)
py_caster = type_caster("@py:", literal_eval)
json_caster = type_caster("@json:", json.loads)
toml_caster = type_caster("@toml:", _cast_toml)


def cast_value(value: Any, casters: Sequence[Callable]) -> Any:
    """Cast a single value"""
    if not isinstance(value, str):
        return value
    for caster in casters:
        try:
            return caster(value)
        except Exception:
            continue
    return value


def cast(conf: "Diot", casters: Sequence[Callable]) -> "Diot":
    """Cast the configuration"""
    for key, value in conf.items():
        if isinstance(value, dict):
            conf[key] = cast(value, casters)
        else:
            conf[key] = cast_value(value, casters)
    return conf



import warnings
from typing import Any
from pathlib import Path
from simpleconf.utils import require_package
from diot import Diot

from ..caster import (
    cast,
    int_caster,
    float_caster,
    bool_caster,
    none_caster,
    python_caster,
    py_caster,
    json_caster,
    toml_caster,
)
from . import Loader

iniconfig = require_package("iniconfig")


class IniLoader(Loader):
    """Ini-like file loader"""

    CASTERS = [
        int_caster,
        float_caster,
        bool_caster,
        none_caster,
        python_caster,
        py_caster,
        json_caster,
        toml_caster,
    ]

    def loading(self, conf: Any, ignore_nonexist: bool) -> dict:
        """Load the configuration from an ini-like file"""
        if not self._exists(conf, ignore_nonexist):
            return Diot(default={})
        return iniconfig.IniConfig(conf).sections

    def load(self, conf: Any, ignore_nonexist: bool = False) -> "Diot":
        """Load and cast the configuration from an ini-like file"""
        sections = self.loading(conf, ignore_nonexist)
        keys = list(sections)
        # only load the default section
        if len(keys) > 1:
            warnings.warn(
                f"{Path(conf).name}: More than one section found, "
                "only the default section will be loaded."
            )

        if len(keys) == 0 or keys[0].lower() != "default":
            raise ValueError(
                f"{Path(conf).name}: Only the default section can be loaded."
            )

        return cast(Diot(sections[keys[0]]), self.__class__.CASTERS)

    def load_with_profiles(
        self,
        conf: Any,
        ignore_nonexist: bool = False,
    ) -> Diot:
        """Load and cast the configuration from an ini-like file with profiles
        """
        sections = self.loading(conf, ignore_nonexist)
        out = Diot()
        for k, v in sections.items():
            out[k.lower()] = cast(v, self.__class__.CASTERS)
        return out



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['simpleconf', 'simpleconf.loaders']

package_data = \
{'': ['*']}

install_requires = \
['diot>=0.1,<0.2']

extras_require = \
{'all': ['python-dotenv>=0.20,<0.21',
         'pyyaml>=6,<7',
         'rtoml>=0.8,<0.9',
         'iniconfig>=1.1,<2.0'],
 'env': ['python-dotenv>=0.20,<0.21'],
 'ini': ['iniconfig>=1.1,<2.0'],
 'toml': ['rtoml>=0.8,<0.9'],
 'yaml': ['pyyaml>=6,<7']}

setup_kwargs = {
    'name': 'python-simpleconf',
    'version': '0.5.7',
    'description': 'Simple configuration management with python.',
    'long_description': '# simpleconf\n\nSimple configuration management for python\n\n## Installation\n```shell\n# released version\npip install python-simpleconf\n\n# Install support for ini\npip install python-simpleconf[ini]\n\n# Install support for dotenv\npip install python-simpleconf[dotenv]\n\n# Install support for yaml\npip install python-simpleconf[yaml]\n\n# Install support for toml\npip install python-simpleconf[toml]\n\n# Install support for all supported formats\npip install python-simpleconf[all]\n```\n\n## Features\n- Multiple formats supported\n- Type casting\n- Profile support\n- Simple APIs\n\n## Usage\n\n### Loading configurations\n\n```python\nfrom simpleconf import Config\n\n# Load a single file\nconf = Config.load(\'~/xxx.ini\')\n# load multiple files, later files override previous ones\nconf = Config.load(\n   \'~/xxx.ini\', \'~/xxx.env\', \'~/xxx.yaml\', \'~/xxx.toml\',\n   \'~/xxx.json\', \'simpleconf.osenv\', {\'a\': 3}\n)\n\n# Load a single file with a different loader\nconf = Config.load(\'~/xxx.ini\', loader="toml")\n```\n\n### Accessing configuration values\n\n```python\nfrom simpleconf import Config\n\nconf = Config.load({\'a\': 1, \'b\': {\'c\': 2}})\n# conf.a == 1\n# conf.b.c == 2\n```\n\n### Supported formats\n\n- `.ini/.cfg/.config` (parsed by `iniconfig`).\n  - For confiurations without profiles, an ini-like configuration like must have a `default` (case-insensitive) section.\n- `.env` (using `python-dotenv`). A file with environment variables.\n- `.yaml/.yml` (using `pyyaml`). A file with YAML data.\n- `.toml` (using `rtoml`). A file with TOML data.\n- `.json` (using `json`). A file with JSON data.\n- `XXX.osenv`: System environment variables with prefix `XXX_` (case-sensitive) is used.\n  - `XXX_A=1` will be loaded as `conf.A = 1`.\n- python dictionary.\n\n### Profile support\n\n#### Loading configurations\n\n##### Loading dictionaries\n\n```python\nfrom simpleconf import ProfileConfig\n\nconf = ProfileConfig.load({\'default\': {\'a\': 1})\n# conf.a == 1\n```\n\n##### Loading a `.env` file\n\n`config.env`\n```env\n# config.env\ndefault_a=1\n```\n\n```python\nfrom simpleconf import ProfileConfig\n\nconf = ProfileConfig.load(\'config.env\')\n# conf.a == 1\n```\n\n##### Loading ini-like configuration files\n\n```ini\n# config.ini\n[default]\na = 1\n```\n\n```python\nfrom simpleconf import ProfileConfig\n\nconf = ProfileConfig.load(\'config.ini\')\n# conf.a == 1\n```\n\n##### Loading JSON files\n\n`config.json`\n```json\n{\n  "default": {\n    "a": 1\n  }\n}\n```\n\n```python\nfrom simpleconf import ProfileConfig\n\nconf = ProfileConfig.load(\'config.json\')\n# conf.a == 1\n```\n\n##### Loading system environment variables\n\n```python\nfrom os import environ\nfrom simpleconf import ProfileConfig\n\nenviron[\'XXX_DEFAULT_A\'] = \'1\'\n\nconf = ProfileConfig.load(\'XXX.osenv\')\n# conf.a == 1\n```\n\n##### Loading TOML files\n\n```toml\n# config.toml\n[default]\na = 1\n```\n\n```python\nfrom simpleconf import ProfileConfig\n\nconf = ProfileConfig.load(\'config.toml\')\n# conf.a == 1\n```\n\n##### Loading YAML files\n\n```yaml\n# config.yaml\ndefault:\n  a: 1\n```\n\n```python\nfrom simpleconf import ProfileConfig\n\nconf = ProfileConfig.load(\'config.yaml\')\n# conf.a == 1\n```\n\n#### Switching profile\n\n```python\nfrom simpleconf import ProfileConfig\n\nconf = ProfileConfig.load(\n   {\'default\': {\'a\': 1, \'b\': 2}, \'dev\': {\'a\': 3}, \'prod\': {\'a\': 4}}\n)\n# conf.a == 1; conf.b == 2\n# ProfileConfig.profiles(conf) == [\'default\', \'dev\', \'prod\']\n# ProfileConfig.pool(conf) == {\'default\': {\'a\': 1, \'b\': 2}, \'dev\': {\'a\': 3}, \'prod\': {\'a\': 4}}\n# ProfileConfig.current_profile(conf) == \'default\'\n# ProfileConfig.base_profile(conf) == \'default\'\n\nProfileConfig.use_profile(conf, \'dev\')\n# conf.a == 3; conf.b == 2\n# ProfileConfig.current_profile(conf) == \'dev\'\n# ProfileConfig.base_profile(conf) == \'default\'\n\n# use a different base profile\nProfileConfig.use_profile(conf, \'prod\', base=\'dev\')\n# conf.a == 4   # No \'b\' in conf\n# ProfileConfig.current_profile(conf) == \'prod\'\n# ProfileConfig.base_profile(conf) == \'dev\'\n\n# Copy configuration instead of inplace modification\nconf2 = ProfileConfig.use_profile(conf, \'dev\', copy=True)\n# conf2 is not conf\n# conf2.a == 3; conf2.b == 2\n\n# Use a context manager\nwith ProfileConfig.use_profile(conf2, \'default\'):\n    conf2.a == 3\n    conf2.b == 2\n# conf2.a == 3; conf2.b == 2\n```\n\n### Type casting\n\nFor configuration formats with type support, including dictionary, no type casting is done by this library, except that for TOML files.\n\nTOML does not support `None` value in python. We use `rtoml` library to parse TOML files, which dumps `None` as `"null"`. So a `null_caster` is used to cast `"null"` to `None`.\n\nA `none_caster` is also enabled for TOML files, a pure string of `"@none"` is casted to `None`.\n\nFor other formats, following casters are supported:\n\n#### Int caster\n\n```python\nfrom os import environ\nfrom simpleconf import Config\n\nenviron[\'XXX_A\'] = \'@int:1\'\n\nconf = Config.load(\'XXX.osenv\')\n# conf.a == 1 # int\n```\n\n#### Float caster\n\n`@float:1.0` -> `1.0`\n\n### Bool caster\n\n`@bool:true` -> `True`\n`@bool:false` -> `False`\n\n#### Python caster\n\nValues are casted by `ast.literal_eval()`.\n\n```python\n"@python:1" => 1  # or\n"@py:1" => 1\n"@py:1.0` -> `1.0`\n"@py:[1, 2, 3]" => [1, 2, 3]\n```\n\n#### JSON caster\n\n`@json:{"a": 1}` -> `{"a": 1}`\n\n#### TOML caster\n\n`@toml:a = 1` -> `{"a": 1}`\n',
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pwwang/simpleconf',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
